@@ -435,8 +435,8 @@ class Chain:
 
         :param match: optional :class:`Match` object; use a :class:`MatchNone`
             object to delete a rule that has no matches
-        :param target: optional :class:`Target` object; use a :class:`TargetNone`
-            object to delete a rule that has no target
+        :param target: optional :class:`Target` object; use a
+            :class:`TargetNone` object to delete a rule that has no target
         """
         if match is None and target is None:
             return 0
@@ -452,6 +452,13 @@ class Chain:
         """
         return self.delete_rule_by_pred(pred=lambda r: r.targets_chain(chain))
 
+    def zero_counters(self) -> None:
+        """Zero the packet and byte counters of this chain in the kernel.
+        """
+        if self.__pft is None:
+            raise IptablesError('chain not in kernel')
+        self.__pft.zero_counters(chain=self)
+
     @classmethod
     def __parse_chain_line(cls, line) -> 'Chain':
         """Parse a line which has one of the following 2 forms:
