@@ -20,7 +20,4 @@
 
 # pylint: disable=unused-import
 
-try:
-    from progutil.logging import get_logger
-except ImportError:
-    from logging import getLogger as get_logger
+from logging import getLogger as get_logger
