@@ -1,4 +1,4 @@
-# Copyright (c) 2021, 2022, Panagiotis Tsirigotis
+# Copyright (c) 2021, 2022, 2023, Panagiotis Tsirigotis
 
 # This file is part of linuxnet-iptables.
 #
@@ -18,145 +18,20 @@
 """This module provides the ChainRule class
 """
 
-from collections import deque
 from typing import List, Optional
 
 from .exceptions import IptablesError, IptablesParsingError
-from .match import (
-                        parse_criteria,
-                        Match,
-                        MatchNone,
-                        PacketMatch,
-                    )
-from .target import (
-                        parse_target,
-                        Target,
-                        ChainTarget,
-                        UnparsedTarget,
-                        TargetNone,
-                    )
+from .matches import Match, MatchNone, PacketMatch
+from .matches.match import MatchParser
+from .parsing import LookaheadIterator, RuleFieldIterator
+from .targets import Target, ChainTarget, UnparsedTarget, TargetNone
+from .targets.target import TargetParser
 from .deps import get_logger
 
 
 _logger = get_logger("linuxnet.iptables.rule")
 
 
-class _LookaheadIterator:
-    """A LookaheadIterator provides the ability to put back previously
-    returned tokens.
-
-    Conceptual view of LookaheadIterator:
-
-                          deque
-       +----------+  +---+---+---+---+---+
-       | Iterator |  | T | T | T |...| T |
-       +----------+  +---+---+---+---+---+
-                           ^
-                           |
-                         Cursor
-    Tokens to the right of the cursor have been consumed.
-    Tokens up to, but not including, the cursor are previously consumed
-    tokens that have been put back.
-    The value of the cursor indicates the number of put-back tokens
-    The maximum size of the deque is equal to the lookahead.
-    """
-    def __init__(self, iterable, lookahead: int):
-        self.__iter = iter(iterable)
-        if lookahead <= 0:
-            raise ValueError(f'bad lookahead value {lookahead}')
-        self.__tokens = deque(maxlen=lookahead)
-        self.__cursor = 0
-
-    def __iter__(self):
-        return self
-
-    def __next__(self):
-        if self.__cursor == 0:
-            token = next(self.__iter)
-            self.__tokens.appendleft(token)
-        else:
-            self.__cursor -= 1
-            token = self.__tokens[self.__cursor]
-        return token
-
-    def peek(self) -> Optional[str]:
-        """Returns the next token, but does not consume it
-        """
-        try:
-            token = self.__next__()
-            self.put_back(token)
-            return token
-        except StopIteration:
-            return None
-
-    def put_back(self, token: str) -> None:
-        """Put back the specified token. This must be a token previously
-        returned by the iterator (identity is checked, not equality)
-        """
-        if self.__cursor == len(self.__tokens):
-            # Either there are no consumed tokens, or this is an attempt to
-            # put back one more token than those already consumed.
-            raise ValueError('not a consumed token')
-        if token is not self.__tokens[self.__cursor]:
-            raise ValueError('wrong token')
-        self.__cursor += 1
-
-    def rewind(self, step=1) -> None:
-        """Putback <step> tokens
-        """
-        avail = len(self.__tokens) - self.__cursor
-        if step > avail:
-            raise ValueError(f'unable to rewind {step} token(s)')
-        self.__cursor += step
-
-
-class _CriteriaTargetIterator(_LookaheadIterator):
-    """The purpose of this iterator is to record the criteria
-    fields that we were unable to parse.
-    """
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.__stored_fields = []
-
-    def forward_past(self, field_name: str) -> None:
-        """Forward past the specified field, which must exist.
-        """
-        try:
-            while True:
-                field = next(self)
-                if field == field_name:
-                    return
-                self.__stored_fields.append(field)
-        except StopIteration as stopit:
-            raise IptablesParsingError(
-                        f"missing '{field_name}'") from stopit
-
-    def next_value(self, attr: str) -> str:
-        """Helper method to return the next field which holds the 'value'
-        for attr
-        """
-        try:
-            return next(self)
-        except StopIteration as stopit:
-            raise IptablesParsingError(
-                        f"missing value for '{attr}'") from stopit
-
-    def store_field(self, field: str) -> None:
-        """Store the specified field
-        """
-        self.__stored_fields.append(field)
-
-    def store_rest(self) -> None:
-        """Store any remaining fields
-        """
-        self.__stored_fields.extend(self)
-
-    def get_stored_fields(self) -> List[str]:
-        """Returns any stored fields
-        """
-        return self.__stored_fields
-
-
 class ChainRule:        # pylint: disable=too-many-instance-attributes
     """This class represents a rule in an **iptables(8)** chain.
     A :class:`ChainRule` has a (possibly empty) list of :class:`Match`
@@ -433,7 +308,7 @@ class ChainRule:        # pylint: disable=too-many-instance-attributes
                                 'esp', 'ah', 'sctp', 'all'))
 
     @classmethod
-    def __parse_target_name(cls, field_iter: _LookaheadIterator):
+    def __parse_target_name(cls, field_iter: LookaheadIterator):
         """Returns the target_name if present, otherwise None
         """
         #
@@ -476,7 +351,7 @@ class ChainRule:        # pylint: disable=too-many-instance-attributes
         fields = iptables_output_line.split()
         # Minimum lookahead is 3, in order to support put_back based
         # on peek'ed value
-        field_iter = _LookaheadIterator(fields, 3)
+        field_iter = LookaheadIterator(fields, 3)
         try:
             packet_count = int(next(field_iter))
             byte_count = int(next(field_iter))
@@ -489,9 +364,11 @@ class ChainRule:        # pylint: disable=too-many-instance-attributes
             if field_iter.peek() == '[goto]':
                 _ = next(field_iter)
                 uses_goto = True
-            match_list = parse_criteria(field_iter)
-            new_field_iter = _CriteriaTargetIterator(field_iter, 3)
-            target = parse_target(target_name, new_field_iter, uses_goto)
+            parser = MatchParser(field_iter)
+            match_list = parser.parse_matches()
+            new_field_iter = RuleFieldIterator(field_iter, 3)
+            parser = TargetParser(target_name, new_field_iter)
+            target = parser.parse_target(uses_goto)
             stored_fields = new_field_iter.get_stored_fields()
             if stored_fields:
                 raise IptablesParsingError(
