@@ -78,7 +78,7 @@ class IptablesPacketFilterTable:
 
     __CHAIN_NAME_LIMIT = 28
 
-    def __init__(self, table_name: str, chain_prefix: str =None, runner=_run):
+    def __init__(self, table_name: str, chain_prefix: str =None, runner=None):
         """
         :param table_name: one of ``filter``, ``mangle``, ``nat``, ``raw``
         :param chain_prefix: if specified, only chains with names starting
@@ -98,7 +98,7 @@ class IptablesPacketFilterTable:
             raise IptablesError(f"unknown table '{table_name}'")
         self.__prefix = chain_prefix
         self.__table_name = table_name
-        self.__runner = runner
+        self.__runner = runner or _run
         # If chain_prefix is not None, the __chain_map will contain
         # only chains whose name starts with this prefix, otherwise
         # it will contain all chains.
@@ -644,4 +644,22 @@ class IptablesPacketFilterTable:
         self.__chain_map.pop(real_chain_name, None)
         chain.clear_pft()
 
+    def zero_counters(self, chain: Optional[Chain] =None) -> None:
+        """Zero the packet and byte counters for the specified chain
+        in the kernel. If no chain is specified, all chain counters are
+        zero'd.
+
+        Note that this method does not affect the counters of :class:`Chain`
+        objects which will maintain the values they had from the last time
+        the system configuration was read.
+        """
+        cmd = ['-Z']
+        if chain is not None:
+            if chain.get_pft() is not self:
+                raise IptablesError(
+                    'attempt to zero chain counters of '
+                    'chain owned by another table')
+            cmd.append(chain.get_real_name())
+        _ = self.iptables_run(cmd, check=True)
+
 # pylint: enable=too-many-instance-attributes, too-many-public-methods
