@@ -24,6 +24,7 @@ import subprocess
 import unittest
 import sys
 
+from collections import namedtuple
 from ipaddress import IPv4Network, IPv4Address
 
 curdir = os.getcwd()
@@ -51,6 +52,7 @@ from linuxnet.iptables import (
                 PacketTypeMatch,
                 ConnmarkMatch,
                 LimitMatch,
+                OwnerMatch,
                 )
 
 root_logger = logging.getLogger()
@@ -58,14 +60,29 @@ root_logger.addHandler(logging.FileHandler('test.log', 'w'))
 root_logger.setLevel(logging.INFO)
 
 
+ExecutedCommand = namedtuple('ExecutedCommand', ['cmd', 'args', 'kwargs'])
+
+
 class SimulatedIptablesRun:     # pylint: disable=too-few-public-methods
     """Simulate a run of iptables
     """
     def __init__(self, exitcode, output):
         self.__output = output
         self.__exitcode = exitcode
+        # A run is a list of ExecutedCommand instances
+        self.__run = []
 
-    def __call__(self, *args, **kwargs):
+    def get_cmd(self):
+        return self.__cmd
+
+    def get_run(self):
+        return self.__run
+
+    def clear_run(self):
+        self.__run.clear()
+
+    def __call__(self, cmd, *args, **kwargs):
+        self.__run.append(ExecutedCommand(cmd, args, kwargs))
         proc = subprocess.CompletedProcess(args, self.__exitcode)
         proc.stdout = self.__output
         return proc
@@ -365,6 +382,84 @@ Chain INPUT (policy DROP 0 packets, 0 bytes)
         self.assertTrue(icmp_type.is_positive() and
                         icmp_type.get_type_value() == 3)
 
+    def test_parsing_owner_match(self):
+        """Parse output with match related to UID/GID
+        """
+        output = self.EMPTY_INPUT + '\n' + self.EMPTY_FORWARD + '\n' + \
+"""\
+Chain OUTPUT (policy DROP 0 packets, 0 bytes)
+    pkts      bytes target     prot opt in     out     source               destination
+       8      524   DROP       all  --  *      *       0.0.0.0/0            0.0.0.0/0           owner UID match 100
+       8      524   DROP       all  --  *      *       0.0.0.0/0            0.0.0.0/0           ! owner GID match 100
+       8      524   DROP       all  --  *      *       0.0.0.0/0            0.0.0.0/0           owner UID match 100-300 ! owner UID match 200
+       8      524   DROP       all  --  *      *       0.0.0.0/0            0.0.0.0/0           owner socket exists 
+       8      524   DROP       all  --  *      *       0.0.0.0/0            0.0.0.0/0           owner GID match 100 ! owner socket exists 
+"""
+        pft = IptablesPacketFilterTable('filter')
+        self.assertTrue(pft.init_from_output(output))
+        output_chain = pft.get_builtin_chain('OUTPUT')
+        rules = output_chain.get_rules()
+        self.assertEqual(len(rules), 5)
+        #
+        # Verify first rule
+        #
+        rule = rules[0]
+        match_list = rule.get_match_list()
+        self.assertEqual(len(match_list), 1)
+        match = match_list[-1]
+        self.assertTrue(isinstance(match, OwnerMatch))
+        uid = match.uid()
+        self.assertTrue(uid.is_positive() and uid.get_value() == (100, None))
+        #
+        # Verify second rule
+        #
+        rule = rules[1]
+        match_list = rule.get_match_list()
+        self.assertEqual(len(match_list), 1)
+        match = match_list[-1]
+        self.assertTrue(isinstance(match, OwnerMatch))
+        gid = match.gid()
+        self.assertTrue(not gid.is_positive() and gid.get_value() == (100, None))
+        #
+        # Verify third rule
+        #
+        rule = rules[2]
+        match_list = rule.get_match_list()
+        self.assertEqual(len(match_list), 2)
+        match = match_list[-2]
+        self.assertTrue(isinstance(match, OwnerMatch))
+        uid = match.uid()
+        self.assertTrue(uid.is_positive() and uid.get_value() == (100, 300))
+        match = match_list[-1]
+        self.assertTrue(isinstance(match, OwnerMatch))
+        uid = match.uid()
+        self.assertTrue(not uid.is_positive() and uid.get_value() == (200, None))
+        #
+        # Verify fourth rule
+        #
+        rule = rules[3]
+        match_list = rule.get_match_list()
+        self.assertEqual(len(match_list), 1)
+        match = match_list[-1]
+        self.assertTrue(isinstance(match, OwnerMatch))
+        socket_exists = match.socket_exists()
+        self.assertTrue(socket_exists.is_positive())
+        #
+        # Verify fifth rule
+        # Note that a single match is expected because the owner
+        # match criteria are different.
+        #
+        rule = rules[4]
+        match_list = rule.get_match_list()
+        self.assertEqual(len(match_list), 1)
+        match = match_list[-1]
+        self.assertTrue(isinstance(match, OwnerMatch))
+        gid = match.gid()
+        socket_exists = match.socket_exists()
+        self.assertTrue(
+                gid.is_positive() and gid.get_value() == (100, None) and
+                not socket_exists.is_positive())
+
     def test_parsing_connmark_match(self):
         """Parse output with connmark match
         """
@@ -832,6 +927,30 @@ Chain prod_OUTPUT (1 references)
             self.assertTrue(init_ok)
             self.assertEqual(pft.get_epoch(), i)
 
+    def test_zero_counters(self):
+        """Test that the correct command is issued to zero counters
+        """
+        runner = SimulatedIptablesRun(0, None)
+        pft = IptablesPacketFilterTable('filter', runner=runner)
+        init_ok = pft.init_from_output(self.GOOD_OUTPUT)
+        self.assertTrue(init_ok)
+        chain = pft.get_builtin_chain('OUTPUT')
+        #
+        # Zero the counters of a specific chain
+        #
+        chain.zero_counters()
+        expected_cmd = ['iptables', '-t', 'filter', '-Z', 'OUTPUT']
+        commands = runner.get_run()
+        self.assertEqual(commands[0].cmd, expected_cmd)
+        #
+        # Zero the counters of all chains
+        #
+        runner.clear_run()
+        pft.zero_counters()
+        commands = runner.get_run()
+        expected_cmd = ['iptables', '-t', 'filter', '-Z']
+        self.assertEqual(commands[0].cmd, expected_cmd)
+
 
 class TestChainOperations(unittest.TestCase):
     """Test chain operations. The runner is a no-op
